@@ -30,6 +30,7 @@
 #include <QSqlDatabase>
 #include <QSqlError>
 #include <QVariant>
+#include <QList>
 
 #include "globalconstants.h"
 #include "chatlogger.h"
@@ -37,7 +38,7 @@
 #include "Whatsapp/util/utilities.h"
 
 #define MAX_MESSAGES               7
-#define LOG_VERSION                6
+#define LOG_VERSION                7
 #define LOG_EXTENSION              ".dblog"
 
 // Column definitions
@@ -65,6 +66,11 @@
 #define LOG_MSG_COUNT               20
 #define LOG_MSG_DELIVERED           21
 #define LOG_MSG_READ                22
+#define LOG_MSG_REMOTE_RESOURCE     23
+
+#define LOG_RECEIPT_REMOTE_JID       1
+#define LOG_RECEIPT_TYPE             2
+#define LOG_RECEIPT_TIMESTAMP        3
 
 ChatLogger::ChatLogger(QObject *parent):
     QObject(parent)
@@ -124,9 +130,17 @@ bool ChatLogger::init(QString jid)
                    "media_caption varchar(160),"
                    "msg_count integer,"
                    "msg_delivered integer,"
-                   "msg_read integer"
+                   "msg_read integer,"
+                   "remote_resource varchar(20)"
                    ")");
 
+        query.exec("create table receipt ("
+                   "message_id integer not null,"
+                   "participant varchar(20) not null,"
+                   "type integer not null,"
+                   "timestamp integer not null,"
+                   "unique(message_id, participant, type))");
+
         query.exec("create table settings ("
                    "version integer"
                    ")");
@@ -188,7 +202,6 @@ bool ChatLogger::init(QString jid)
 
             case 5:
                 // Upgrade it to version 6
-
                 Utilities::logData("Upgrading log " + jid + " to version " +
                                    QString::number(LOG_VERSION));
 
@@ -221,6 +234,19 @@ bool ChatLogger::init(QString jid)
                            ")");
                 query.exec("insert into log select * from tmp_log");
                 query.exec("drop table tmp_log");
+
+            case 6:
+                // Upgrade it to version 7
+                Utilities::logData("Upgrading log " + jid + " to version " +
+                                   QString::number(LOG_VERSION));
+
+		query.exec("create table receipt ("
+                           "message_id integer not null,"
+                           "participant varchar(20) not null,"
+                           "type integer not null,"
+			   "timestamp integer not null,"
+			   "unique(message_id, participant, type))");
+                query.exec("alter table log add remote_resource varchar(20)");
             }
             query.exec("update settings set version=" +
                         QString::number(LOG_VERSION));
@@ -243,18 +269,19 @@ void ChatLogger::logMessage(FMessage message)
 {
     QSqlQuery query(db);
 
+    query.exec("BEGIN TRANSACTION");
     query.prepare("insert into log (name, from_me, timestamp, "
                   "id, type, data, thumb_image, status, "
                   "media_url, media_mime_type, media_wa_type, media_size, "
                   "media_name, media_duration_seconds, local_file_uri,"
                   "live, latitude, longitude, media_caption,"
-                  "msg_count, msg_delivered, msg_read)"
+                  "msg_count, msg_delivered, msg_read, remote_resource)"
                   "values (:name, :from_me, :timestamp, :id, "
                   ":type, :data, :thumb_image, :status, "
                   ":media_url, :media_mime_type, :media_wa_type, :media_size, "
                   ":media_name, :media_duration_seconds, :local_file_uri,"
                   ":live, :latitude, :longitude, :media_caption,"
-                  ":msg_count, :msg_delivered, :msg_read"
+                  ":msg_count, :msg_delivered, :msg_read, :remote_resource"
                   ")");
 
     query.bindValue(":name",message.notify_name);
@@ -287,9 +314,49 @@ void ChatLogger::logMessage(FMessage message)
     query.bindValue(":msg_count", message.count);
     query.bindValue(":msg_delivered", message.delivered);
     query.bindValue(":msg_read", message.read);
+
+    query.bindValue(":remote_resource", message.remote_resource);
     query.exec();
 
+    query.exec("select max(localid) from log");
+    query.next();
+    int log_id = query.value(0).toInt();
+
     Utilities::logData("logMessage(): " + query.lastError().text());
+
+    for(int i=0; i<message.receipts.size(); i++)
+    {
+        Utilities::logData("INSERTING receipt");
+        query.prepare("insert into receipt (message_id, participant, type, timestamp) "
+                      "values(:message_id, :participant, :type, :timestamp)");
+        query.bindValue(":message_id,", log_id);
+        query.bindValue(":participant", message.receipts.at(i).remote_jid);
+        query.bindValue(":type", message.receipts.at(i).type);
+        query.bindValue(":timestamp", message.receipts.at(i).timestamp);
+        query.exec();
+    }
+
+    query.exec("END TRANSACTION");
+}
+
+int ChatLogger::getMessageReceipts(QSqlQuery& query, FMessage &msg)
+{
+    int receipts=0;
+    int localid = query.value(LOG_LOCALID).toInt();
+
+    query.prepare("select * from receipt "
+                  "where message_id = :message_id "),
+    query.bindValue(":message_id",localid);
+    query.exec();
+
+    while (query.next()) {
+        Receipt r(query.value(LOG_RECEIPT_REMOTE_JID).toString(),
+                  (Receipt::ReceiptType)query.value(LOG_RECEIPT_TYPE).toInt(),
+                  query.value(LOG_RECEIPT_TIMESTAMP).toLongLong());
+        msg.receipts.append(r);
+        receipts++;
+    }
+    return receipts;
 }
 
 FMessage ChatLogger::sqlQueryResultToFMessage(QString jid,QSqlQuery& query)
@@ -323,6 +390,7 @@ FMessage ChatLogger::sqlQueryResultToFMessage(QString jid,QSqlQuery& query)
     msg.count = query.value(LOG_MSG_COUNT).toInt();
     msg.read = query.value(LOG_MSG_READ).toInt();
     msg.delivered = query.value(LOG_MSG_DELIVERED).toInt();
+    msg.remote_resource = query.value(LOG_MSG_REMOTE_RESOURCE).toString();
 
 
     if (msg.type == FMessage::MediaMessage)
@@ -330,6 +398,9 @@ FMessage ChatLogger::sqlQueryResultToFMessage(QString jid,QSqlQuery& query)
     else
         msg.setData(query.value(LOG_DATA).toString());
 
+    int localid = query.value(LOG_LOCALID).toInt();
+
+
     return msg;
 }
 
@@ -371,6 +442,8 @@ void ChatLogger::updateLoggedMessage(FMessage message)
 {
     QSqlQuery query(db);
 
+    query.exec("BEGIN TRANSACTION");
+
     query.prepare("update log set status=:status, msg_count=:msg_count, "
                 "msg_delivered=:msg_delivered, msg_read=:msg_read where id=:id");
 
@@ -381,6 +454,31 @@ void ChatLogger::updateLoggedMessage(FMessage message)
     query.bindValue(":msg_read",message.read);
 
     query.exec();
+
+    Utilities::logData("logMessage(): " + query.lastError().text());
+
+    query.prepare("select localid from log where id=:id");
+    query.bindValue(":id",message.key.id);
+    query.exec();
+
+    query.next();
+    int log_id = query.value(0).toInt();
+
+    for(int i=0; i<message.receipts.size(); i++)
+    {
+        Utilities::logData("UPDATING receipts");
+        query.prepare("insert or replace into receipt (message_id, participant, type, timestamp) "
+                      "values(:message_id, :participant, :type, :timestamp)");
+        query.bindValue(":message_id,", log_id);
+        query.bindValue(":participant", message.receipts.at(i).remote_jid);
+        query.bindValue(":type", message.receipts.at(i).type);
+        query.bindValue(":timestamp", message.receipts.at(i).timestamp);
+        query.exec();
+    }
+
+    Utilities::logData("logMessage(): " + query.lastError().text());
+
+    query.exec("END TRANSACTION");
 }
 
 void ChatLogger::updateUriMessage(FMessage message)
@@ -443,6 +541,7 @@ FMessage ChatLogger::lastMessage()
 
     if (query.next()) {
         msg = sqlQueryResultToFMessage(jid,query);
+        getMessageReceipts(query, msg);
     } else {
         QString s = "no previous message";
         msg.setData(s);
@@ -476,6 +575,7 @@ FMessage ChatLogger::lastMessage(QString jid)
 
         if (query.next()) {
             msg = sqlQueryResultToFMessage(jid,query);
+            getMessageReceipts(query, msg);
         } else {
             QString s = "no previous message";
             msg.setData(s);
@@ -486,5 +586,3 @@ FMessage ChatLogger::lastMessage(QString jid)
 
     return msg;
 }
-
-
